# coding: utf-8

from __future__ import absolute_import

from flask import json
from six import BytesIO

from swagger_server.models.entity import Entity  # noqa: E501
from swagger_server.test import BaseTestCase


class TestIdController(BaseTestCase):
    """IdController integration test stubs"""

    def test_id_id_get(self):
        """Test case for id_id_get

        Get MDB entity with specified nanoid
        """
        response = self.client.open(
            '/v1/id/{id}'.format(id='id_example'),
            method='GET')
        self.assert200(response,
                       'Response body is : ' + response.data.decode('utf-8'))


if __name__ == '__main__':
    import unittest
    unittest.main()



import connexion
import six

from swagger_server import util


def tags_count_get():  # noqa: E501
    """Get number of tags present in MDB

     # noqa: E501


    :rtype: object
    """
    return 'do some magic!'


def tags_get():  # noqa: E501
    """Get all tag nodes in MDB

     # noqa: E501


    :rtype: object
    """
    return 'do some magic!'



import sys
import re
from flask import (
    render_template,
    redirect,
    url_for,
    request,
    g,
    json,
    jsonify,
    current_app,
    Response,
    abort,
)
from werkzeug.exceptions import HTTPException
from . import bp
from ..mdb import mdb
# sys.path.insert(0,"/Users/jensenma/Code/bento-meta/python")  # noqa E231

from bento_meta.util.makeq import Query

allowed_meths = ['GET', 'POST']

ents = {
    "model": "models",
    "node": "nodes",
    "prop": "props",
    "term": "terms",
    "edge": "edges",
    }

@bp.errorhandler(HTTPException)
def handle_exception(e):
    response = Response(
        status = e.code or 500,
        response=json.dumps({
            "code": e.code or 500,
            "description": str(e.description),
            })
        )
    response.content_type = "application/json"
    response.access_control_allow_origin = "*"
    return response

@bp.before_app_request
def before_request():
    g.locale = 'EN_US'


@bp.route('/', methods=['GET'])
@bp.route('/v1', methods=['GET'])
def index():
    response = jsonify({
        "application": "STS",
        "version": "0.1",
        "status": "READY"
    })
    response.access_control_allow_origin = "*"
    return response



@bp.route("/v1/<path:path>", methods=['GET'])
def query_db(path):
    if not Query.paths:
        Query.set_paths(current_app.config["QUERY_PATHS"])
    current_app.logger.info(path)
    m = mdb()
    skip = request.args.get("skip")
    limit = request.args.get("limit")
    if not limit or limit > current_app.config['MAX_ENTS_PER_REQ']:
        limit = current_app.config['MAX_ENTS_PER_REQ'] 
    q = None
    total_rows = None
    try:
        q = Query(path)
    except Exception as e:
        abort(404, e)
    # look for a paired 'count'
    if not path.endswith('count'):
        try:
            qct = Query(path+"/count")
            ret = m.mdb.get_with_statement(str(qct), qct.params)
            total_rows = list(ret[0].values())[0]
        except Exception as e:
            current_app.logger.warn(e)
            pass
    
    stmt = str(q)
    if skip:
        stmt = stmt + " SKIP {}".format(int(skip))
    if limit:
        stmt = stmt + " LIMIT {}".format(int(limit))
    # kludge tag_entities statement to add label
    if (q.path_id == 'tag_entities'):
        stmt = re.sub("(n([0-9]+) as entities)",
                      "\\1, head(labels(n\\2)) as label",
                      stmt)

    current_app.logger.info(stmt)
    current_app.logger.info(q.params)
    ret = None
    try:
        ret = m.mdb.get_with_statement(stmt, q.params)
    except Exception as e:
        abort(500, "MDB issue: {}".format(str(e)))
    if not ret:
        abort(404, "No results")
    if len(ret) == 1:
        ret = ret[0]
    else:
        if q.path_id == "tag_entities":
            # transform for tagged entities
            for i in range(0,len(ret)):
                ret[i]['entities']['label'] = ret[i]['label']
                del ret[i]['label']
        # standard transform: array of dicts to dict of arrays
        xfm = {}
        keys = ret[0].keys()
        for k in keys:
            xfm[k] = [x[k] for x in ret]
        ret = xfm
        if q.path_id == "tag_values":
            # transform for tag values
            xfm = {
                "key": None,
                "values": {}
                }
            for t in ret['tags']:
                if not xfm['key']:
                    xfm['key'] = t['key']
                if xfm['values'].get(t['value']):
                    xfm['values'][t['value']] += 1
                else:
                    xfm['values'][t['value']] = 1
            ret = xfm

    if total_rows is not None:
        if total_rows > 0:
            ret['count'] = total_rows
        else:
            # ../count was 0
            abort(404,"No items found")
    response = jsonify(ret)
    response.access_control_allow_origin = "*"
    return response



# coding: utf-8

from __future__ import absolute_import

from flask import json
from six import BytesIO

from swagger_server.models.model_property import ModelProperty  # noqa: E501
from swagger_server.models.node import Node  # noqa: E501
from swagger_server.test import BaseTestCase


class TestModelController(BaseTestCase):
    """ModelController integration test stubs"""

    def test_model_model_handle_node_node_handle_get(self):
        """Test case for model_model_handle_node_node_handle_get

        Retrieve a specified node from a model
        """
        response = self.client.open(
            '/v1/model/{modelHandle}/node/{nodeHandle}'.format(modelHandle='modelHandle_example', nodeHandle='nodeHandle_example'),
            method='GET')
        self.assert200(response,
                       'Response body is : ' + response.data.decode('utf-8'))

    def test_model_model_handle_node_node_handle_properties_count_get(self):
        """Test case for model_model_handle_node_node_handle_properties_count_get

        Get number of  properties for specified node
        """
        response = self.client.open(
            '/v1/model/{modelHandle}/node/{nodeHandle}/properties/count'.format(modelHandle='modelHandle_example', nodeHandle='nodeHandle_example'),
            method='GET')
        self.assert200(response,
                       'Response body is : ' + response.data.decode('utf-8'))

    def test_model_model_handle_node_node_handle_properties_get(self):
        """Test case for model_model_handle_node_node_handle_properties_get

        Get all properties for specified node
        """
        query_string = [('skip', 56),
                        ('limit', 56)]
        response = self.client.open(
            '/v1/model/{modelHandle}/node/{nodeHandle}/properties'.format(modelHandle='modelHandle_example', nodeHandle='nodeHandle_example'),
            method='GET',
            query_string=query_string)
        self.assert200(response,
                       'Response body is : ' + response.data.decode('utf-8'))

    def test_model_model_handle_node_node_handle_property_prop_handle_get(self):
        """Test case for model_model_handle_node_node_handle_property_prop_handle_get

        Retrieve a specified property from a model
        """
        response = self.client.open(
            '/v1/model/{modelHandle}/node/{nodeHandle}/property/{propHandle}'.format(modelHandle='modelHandle_example', nodeHandle='nodeHandle_example', propHandle='propHandle_example'),
            method='GET')
        self.assert200(response,
                       'Response body is : ' + response.data.decode('utf-8'))

    def test_model_model_handle_node_node_handle_property_prop_handle_term_term_value_get(self):
        """Test case for model_model_handle_node_node_handle_property_prop_handle_term_term_value_get

        Retrieve a specified term from a property\\'s acceptable value set 
        """
        response = self.client.open(
            '/v1/model/{modelHandle}/node/{nodeHandle}/property/{propHandle}/term/{termValue}'.format(modelHandle='modelHandle_example', nodeHandle='nodeHandle_example', propHandle='propHandle_example', termValue='termValue_example'),
            method='GET')
        self.assert200(response,
                       'Response body is : ' + response.data.decode('utf-8'))

    def test_model_model_handle_node_node_handle_property_prop_handle_terms_count_get(self):
        """Test case for model_model_handle_node_node_handle_property_prop_handle_terms_count_get

        Get number of  properties for specified node
        """
        response = self.client.open(
            '/v1/model/{modelHandle}/node/{nodeHandle}/property/{propHandle}/terms/count'.format(modelHandle='modelHandle_example', nodeHandle='nodeHandle_example', propHandle='propHandle_example'),
            method='GET')
        self.assert200(response,
                       'Response body is : ' + response.data.decode('utf-8'))

    def test_model_model_handle_node_node_handle_property_prop_handle_terms_get(self):
        """Test case for model_model_handle_node_node_handle_property_prop_handle_terms_get

        Get the terms (acceptable values) for specified property, if applicable to property 
        """
        query_string = [('skip', 56),
                        ('limit', 56)]
        response = self.client.open(
            '/v1/model/{modelHandle}/node/{nodeHandle}/property/{propHandle}/terms'.format(modelHandle='modelHandle_example', nodeHandle='nodeHandle_example', propHandle='propHandle_example'),
            method='GET',
            query_string=query_string)
        self.assert200(response,
                       'Response body is : ' + response.data.decode('utf-8'))

    def test_model_model_handle_nodes_count_get(self):
        """Test case for model_model_handle_nodes_count_get

        Get number of nodes for specified model
        """
        response = self.client.open(
            '/v1/model/{modelHandle}/nodes/count'.format(modelHandle='modelHandle_example'),
            method='GET')
        self.assert200(response,
                       'Response body is : ' + response.data.decode('utf-8'))

    def test_model_model_handle_nodes_get(self):
        """Test case for model_model_handle_nodes_get

        Get all nodes for specified model
        """
        query_string = [('skip', 56),
                        ('limit', 56)]
        response = self.client.open(
            '/v1/model/{modelHandle}/nodes'.format(modelHandle='modelHandle_example'),
            method='GET',
            query_string=query_string)
        self.assert200(response,
                       'Response body is : ' + response.data.decode('utf-8'))


if __name__ == '__main__':
    import unittest
    unittest.main()



# coding: utf-8

from __future__ import absolute_import
from datetime import date, datetime  # noqa: F401

from typing import List, Dict  # noqa: F401

from swagger_server.models.base_model_ import Model
from swagger_server import util


class Term(Model):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    def __init__(self, value: str=None, origin_name: str=None, nanoid: str=None, origin_id: str=None, origin_definition: str=None):  # noqa: E501
        """Term - a model defined in Swagger

        :param value: The value of this Term.  # noqa: E501
        :type value: str
        :param origin_name: The origin_name of this Term.  # noqa: E501
        :type origin_name: str
        :param nanoid: The nanoid of this Term.  # noqa: E501
        :type nanoid: str
        :param origin_id: The origin_id of this Term.  # noqa: E501
        :type origin_id: str
        :param origin_definition: The origin_definition of this Term.  # noqa: E501
        :type origin_definition: str
        """
        self.swagger_types = {
            'value': str,
            'origin_name': str,
            'nanoid': str,
            'origin_id': str,
            'origin_definition': str
        }

        self.attribute_map = {
            'value': 'value',
            'origin_name': 'origin_name',
            'nanoid': 'nanoid',
            'origin_id': 'origin_id',
            'origin_definition': 'origin_definition'
        }

        self._value = value
        self._origin_name = origin_name
        self._nanoid = nanoid
        self._origin_id = origin_id
        self._origin_definition = origin_definition

    @classmethod
    def from_dict(cls, dikt) -> 'Term':
        """Returns the dict as a model

        :param dikt: A dict.
        :type: dict
        :return: The Term of this Term.  # noqa: E501
        :rtype: Term
        """
        return util.deserialize_model(dikt, cls)

    @property
    def value(self) -> str:
        """Gets the value of this Term.


        :return: The value of this Term.
        :rtype: str
        """
        return self._value

    @value.setter
    def value(self, value: str):
        """Sets the value of this Term.


        :param value: The value of this Term.
        :type value: str
        """
        if value is None:
            raise ValueError("Invalid value for `value`, must not be `None`")  # noqa: E501

        self._value = value

    @property
    def origin_name(self) -> str:
        """Gets the origin_name of this Term.


        :return: The origin_name of this Term.
        :rtype: str
        """
        return self._origin_name

    @origin_name.setter
    def origin_name(self, origin_name: str):
        """Sets the origin_name of this Term.


        :param origin_name: The origin_name of this Term.
        :type origin_name: str
        """
        if origin_name is None:
            raise ValueError("Invalid value for `origin_name`, must not be `None`")  # noqa: E501

        self._origin_name = origin_name

    @property
    def nanoid(self) -> str:
        """Gets the nanoid of this Term.


        :return: The nanoid of this Term.
        :rtype: str
        """
        return self._nanoid

    @nanoid.setter
    def nanoid(self, nanoid: str):
        """Sets the nanoid of this Term.


        :param nanoid: The nanoid of this Term.
        :type nanoid: str
        """
        if nanoid is None:
            raise ValueError("Invalid value for `nanoid`, must not be `None`")  # noqa: E501

        self._nanoid = nanoid

    @property
    def origin_id(self) -> str:
        """Gets the origin_id of this Term.


        :return: The origin_id of this Term.
        :rtype: str
        """
        return self._origin_id

    @origin_id.setter
    def origin_id(self, origin_id: str):
        """Sets the origin_id of this Term.


        :param origin_id: The origin_id of this Term.
        :type origin_id: str
        """

        self._origin_id = origin_id

    @property
    def origin_definition(self) -> str:
        """Gets the origin_definition of this Term.


        :return: The origin_definition of this Term.
        :rtype: str
        """
        return self._origin_definition

    @origin_definition.setter
    def origin_definition(self, origin_definition: str):
        """Sets the origin_definition of this Term.


        :param origin_definition: The origin_definition of this Term.
        :type origin_definition: str
        """

        self._origin_definition = origin_definition



from flask import current_app
from collections import namedtuple


class mdb_tags:
    """Mixin for reading MDB Tags and customizing UI functionality 
based on them."""

    # ###################################################################### #
    # DATA SUBSETS / TAGS
    # ###################################################################### #

    @staticmethod
    def _get_tags_from_db(tx, model=None):
        result = []

        current_app.logger.warn('yup... using model {}'.format(model))

        query_against_all_models = """
                MATCH (n:node) -[:has_property]->(p:property)-[:has_tag]->(t:tag)
                WHERE n._to IS NULL 
                AND   p._to IS NULL
                RETURN distinct n.handle, n.nanoid, p.handle, p.nanoid, t.key, t.value;
                """
        query_against_specific_model = """
                MATCH (n:node) -[:has_property]->(p:property)-[:has_tag]->(t:tag)
                WHERE n._to IS NULL 
                AND   p._to IS NULL
                WHERE toLower(n.model) = toLower($model)
                RETURN distinct n.handle, n.nanoid, p.handle, p.nanoid, t.key, t.value;
                """

        db_records = None
        if model is None:
            db_records = tx.run(query_against_all_models,)
            current_app.logger.warn(' point 2 ... using model {}'.format(model))
        else:
            db_records = tx.run(query_against_specific_model, model=model)
            current_app.logger.warn(' point 3 ... using model {}'.format(model))
        for record in db_records:
            result.append(record)

        return result

    def get_tags(self, model=None):
        with self.driver.session() as session:
            current_app.logger.warn('have model {}'.format(model))
            tag_records = session.read_transaction(self._get_tags_from_db, model)
            formatted_tags = self.format_tags_records(tag_records)
        return formatted_tags

    def format_tags_records(self, dataset):
        """
        explanation
        iterate, put keys out front, containing an array (for table)
        """
        dict_of_tags = {}
        tagged_record = namedtuple('datatag', ['node', 'node_nanoid', 'property', 'property_nanoid', 'tag_value'])
        for row in dataset:
            tr = tagged_record(row[0], row[1], row[2], row[3], row[5])
            tag = row[4]

            if tag not in dict_of_tags:
                dict_of_tags[tag] = []
            dict_of_tags[tag].append(tr)

        # sort by node
        for tag in dict_of_tags:
            stuff = dict_of_tags[tag]
            sorted_stuff = sorted(stuff, key=lambda x: (x.node, x.property, x.tag_value))
            dict_of_tags[tag] = sorted_stuff
        return dict_of_tags

    def get_tag_keys(self, model=None):
        with self.driver.session() as session:
            current_app.logger.warn('have model {}'.format(model))
            tag_records = session.read_transaction(self._get_tags_from_db, model)
            formatted_tags = self.format_tag_keys(tag_records)
        return formatted_tags

    def format_tag_keys(self, tag_records):
        """
        explanation
        iterate, put keys out front, containing an array (for table)
        """
        tag_keys = []
        #tagged_record = namedtuple('datatag', ['node', 'node_nanoid', 'property', 'property_nanoid', 'tag_value'])
        for row in tag_records:
            tag_key = row[4]
            tag_value = row[5]

            if tag_key not in tag_keys:
                tag_keys.append(tag_key)

        # sort by node
        tag_keys.sort()
        return tag_keys    


    def get_dataset_tags(self, dataset=None, model=None):
        with self.driver.session() as session:
            current_app.logger.warn('have model {}'.format(model))
            tag_records = session.read_transaction(self._get_dataset_tags_from_db, dataset, model)
            formatted_tags = self.format_tags_records(tag_records)
        return formatted_tags


    @staticmethod
    def _get_dataset_tags_from_db(tx, dataset=None, model=None):
        result = []

        current_app.logger.warn('getting for tag {} in model {}'.format(dataset, model))

        dataset_query_ = """
                MATCH (n:node)-[np:has_property]->(p:property)-[:has_tag]->(t:tag)
                WHERE toLower(n.model) = toLower($model)
                AND   t.key = "submitter"
                AND   t.value = $dataset
                AND   n._to IS NULL
                AND   p._to IS NULL
                MATCH (n)-[z:in_dataset { dataset: $dataset } ]->(p)
                RETURN n.handle, n.nanoid, p.handle, p.nanoid, t.key, t.value;
                """

        all_query_ = """
                MATCH (n:node)-[np:has_property]->(p:property)
                WHERE toLower(n.model) = toLower($model)
                AND   n._to IS NULL
                AND   p._to IS NULL
                RETURN n.handle, n.nanoid, p.handle, p.nanoid, n.ONLYthisANDnothingMORE, n.QUOTHtheRAVENnevermore; 
                """
        db_records = None

        if (dataset == 'all') or dataset is None:
            db_records = tx.run(all_query_, dataset=dataset, model=model)
            current_app.logger.warn(' getting entire model {}'.format(model))
            for record in db_records:
                result.append(record)
        else:
            db_records = tx.run(dataset_query_, dataset=dataset, model=model)
            current_app.logger.warn(' getting all tags w/ dataset ... using model {}'.format(model))
            for record in db_records:
                result.append(record)
        return result



    def get_dataset(self, dataset=None, model=None):
        with self.driver.session() as session:
            current_app.logger.warn('have model {}'.format(model))
            tag_records = session.read_transaction(self._get_dataset_from_db, dataset, model)
        return tag_records
            # formatted_tags = self.format_tags_records(tag_records)
        # return formatted_tags

    @staticmethod
    def _get_dataset_from_db(tx, dataset=None, model=None):
        result = []

        current_app.logger.warn('getting for tag {} in model {}'.format(dataset, model))

        dataset_query_ = """
                MATCH (n:node)-[np:has_property]->(p:property)-[:has_tag]->(t:tag)
                WHERE toLower(n.model) = toLower($model)
                AND   t.key = "submitter"
                AND   t.value = $dataset
                AND   n._to IS NULL
                AND   p._to IS NULL
                MATCH (n)-[z:in_dataset { dataset: $dataset } ]->(p)
                OPTIONAL MATCH (p)-[:has_value_set]->(vs:value_set)
                RETURN n.handle, n.nanoid, p.handle, p.nanoid, t.key, t.value, p.value_domain, vs.nanoid, p.required, p.units, p.description, p.instructions, p.internal_comments, p.example ;
                """

        all_query_ = """
                MATCH (n:node)-[np:has_property]->(p:property)
                WHERE toLower(n.model) = toLower($model)
                AND   n._to IS NULL
                AND   p._to IS NULL
                OPTIONAL MATCH (p)-[:has_value_set]->(vs:value_set)
                RETURN n.handle, n.nanoid, p.handle, p.nanoid, n.ONLYthisANDnothingMORE, n.QUOTHtheRAVENnevermore, p.value_domain, vs.nanoid, p.required, p.units, p.description, p.instructions, p.internal_comments, p.example; 
                """


        db_records = None

        if (dataset == 'all') or dataset is None:
            db_records = tx.run(all_query_, dataset=dataset, model=model)
            current_app.logger.warn(' getting entire model {}'.format(model))
            for record in db_records:
                _entry = []
                for part in record:
                    _entry.append(part)
                #result.append(record)
                result.append(_entry)

        else:
            db_records = tx.run(dataset_query_, dataset=dataset, model=model)
            current_app.logger.warn(' getting all tags w/ dataset ... using model {}'.format(model))
            for record in db_records:
                #result.append(record)
                _entry = []
                for part in record:
                    _entry.append(part)
                result.append(_entry)

        return result



    def get_dataset_tag_choices(self):
        with self.driver.session() as session:
            tag_records = session.read_transaction(self._get_dataset_tag_choices_from_db)
            # formatted_tags = self.format_tags_records(tag_records)
        return tag_records

    @staticmethod
    def _get_dataset_tag_choices_from_db(tx, dataset=None, model=None):
        result = ()

        OLD_query_all_models_and_tag_choices = """
            MATCH (n:node)-[np:has_property]->(p:property)-[:has_tag]->(t:tag)
            WHERE t.key = "submitter"
            AND   n._to IS NULL
            AND   p._to IS NULL
            AND EXISTS (np.dataset)
            RETURN distinct n.model, t.value
            ORDER by n.model ASC, t.value ASC
            """

        NEW_query_all_models_and_tag_choices = """
            MATCH (t:tag)
            WHERE t.key = "submitter"
            RETURN distinct t.model, t.value
            ORDER by t.model ASC, t.value ASC
            """

        query_all_models_and_tag_choices = NEW_query_all_models_and_tag_choices

        db_records = tx.run(query_all_models_and_tag_choices)

        # convert into nested tuple structure for optgroup selectfield
        # ('ICDC', (                <-  option
        #    ('NCATS', 'NCATS'),    <-  key, value pairs for selectfield
        #    ('Glioma', 'Glioma'),
        #    ('UBC01', 'UBC01')
        # )),

        # pt 1. organize for optgroup
        temp_results = {}
        for record in db_records:
            mdl = record[0]
            ky = record[1]
            #print(' model is {}'.format(mdl))
            #print(' key is {}'.format(ky))
            mdlky = mdl + "----" + ky
            if mdl not in temp_results:
                temp_results[mdl] = ()
            arglbargl = temp_results[mdl]
            funkmeister = (mdlky, ky)
            temp_results[mdl] = arglbargl + (funkmeister,)

        # pt 2. casting from dict to tuple needed for optgroup
        for m in temp_results:
            flashmasterj = (m, temp_results[m])
            result = result + (flashmasterj,)

        #print('')
        #print('result is {}'.format(result))
        return result

    def get_submitter_tag_choices(self, model=None):
        with self.driver.session() as session:
            tag_records = session.read_transaction(self._get_submitter_tag_choices_from_db, model)
            # formatted_tags = self.format_tags_records(tag_records)
        return tag_records

    # note: there are a couple of approaches for getting tags from the database
    #       one could get them by starting at node, and then going to property
    #       but I'm now trying to also label the tag directly with {model:"ICDC"}
    #       so I can create "empty" datasets
    @staticmethod
    def _get_submitter_tag_choices_from_db(tx, model=None):
        result = ()

        current_app.logger.warn('.yup... using model {}'.format(model))

        query_against_all_models = """
                MATCH (t:tag)
                WHERE t.key = 'submitter'
                RETURN DISTINCT t.model, t.value
                ORDER by t.model ASC, t.value ASC;
                """
        
        query_against_specific_model = """
                MATCH (t:tag)
                WHERE toLower(t.model) = toLower($model)
                AND   t.key = 'submitter'
                RETURN DISTINCT t.model, t.value
                ORDER by t.model ASC, t.value ASC;
                """

        db_records = None
        if model is None:
            db_records = tx.run(query_against_all_models,)
            current_app.logger.warn('.point 22 ... using model {}'.format(model))
        else:
            db_records = tx.run(query_against_specific_model, model=model)
            current_app.logger.warn('.point 23 ... using model {}'.format(model))
        
        # convert into nested tuple structure for optgroup selectfield
        # ('ICDC', (                <-  option
        #    ('NCATS', 'NCATS'),    <-  key, value pairs for selectfield
        #    ('Glioma', 'Glioma'),
        #    ('UBC01', 'UBC01')
        # )),

        # pt 1. organize for optgroup
        temp_results = {}
        for record in db_records:
            mdl = record[0]
            ky = record[1]
            #print(' model is {}'.format(mdl))
            #print(' key is {}'.format(ky))
            mdlky = mdl + "----" + ky
            if mdl not in temp_results:
                temp_results[mdl] = ()
            arglbargl = temp_results[mdl]
            funkmeister = (mdlky, ky)
            temp_results[mdl] = arglbargl + (funkmeister,)

        # pt 2. casting from dict to tuple needed for optgroup
        for m in temp_results:
            flashmasterj = (m, temp_results[m])
            result = result + (flashmasterj,)

        #print('')
        #print(result)
        return result



import logging
from dotenv import load_dotenv
from logging.handlers import RotatingFileHandler
from bento_meta.mdb import MDB
from flask import Flask #, request, current_app
from flask_bootstrap import Bootstrap4
from flask_moment import Moment
from flask_wtf.csrf import CSRFProtect
from bento_sts.config import Config

bootstrap4 = Bootstrap4()
moment = Moment()
csrf = CSRFProtect()
load_dotenv(".env")

logger = logging.getLogger()
def create_app(config_class=Config):
    # see: "Application Object -> About the First Parameter"
    # at https://flask.palletsprojects.com/en/1.1.x/api/#flask.g
    app = Flask(__name__)
    app.config.from_object(config_class)

    """ or set to None for default theme """
    app.config['BOOTSTRAP_BOOTSWATCH_THEME'] = 'spacelab'

    mdb = MDB(app.config["NEO4J_MDB_URI"],
              user=app.config["NEO4J_MDB_USER"],
              password=app.config["NEO4J_MDB_PASS"])
    app.config['MODEL_LIST'] = [x["m"] for x in mdb.get_model_nodes()]
    app.config['MODEL_LIST'].insert(0,{"handle":"All"})

    bootstrap4.init_app(app)
    moment.init_app(app)
    csrf.init_app(app)

    from .main import bp as main_bp
    app.register_blueprint(main_bp)
    from .api import bp as api_bp
    app.register_blueprint(api_bp)
    from .errors import bp as errors_bp
    app.register_blueprint(errors_bp)

    if not app.debug and not app.testing:
        file_handler = RotatingFileHandler(
            app.config['FLASK_LOGFILE'], maxBytes=10240, backupCount=10
        )
        file_handler.setFormatter(
            logging.Formatter(
                "%(asctime)s %(levelname)s: %(message)s"
                " [in %(pathname)s:%(lineno)d]"
            )
        )
        file_handler.setLevel(logging.INFO)
        app.logger.addHandler(file_handler)

        app.logger.setLevel(logging.DEBUG)
        app.logger.info("STS startup")

    return app



# coding: utf-8

import sys
from setuptools import setup, find_packages

NAME = "swagger_server"
VERSION = "1.0.0"

# To install the library, run the following
#
# python setup.py install
#
# prerequisite: setuptools
# http://pypi.python.org/pypi/setuptools

REQUIRES = ["connexion"]

setup(
    name=NAME,
    version=VERSION,
    description="Simple Terminology Server API",
    author_email="mark.jensen@nih.gov",
    url="",
    keywords=["Swagger", "Simple Terminology Server API"],
    install_requires=REQUIRES,
    packages=find_packages(),
    package_data={'': ['swagger/swagger.yaml']},
    include_package_data=True,
    entry_points={
        'console_scripts': ['swagger_server=swagger_server.__main__:main']},
    long_description="""\
    The Simple Terminology Server (STS) exposes elements of data models in an intuitive, consistent way. Data models expressed in the form of a [property graph](https://en.wikipedia.org/wiki/Graph_database#Labeled-property_graph) -- i.e., as nodes, relationships, properties, and terms -- can be explored and queried via this interface. Data models are stored in an instance of a [Metamodel Database (MDB)](https://github.com/CBIIT/bento-meta) backed by a [Neo4j](https://neo4j.com) server.
    """
)




import pprint
from flask import url_for, current_app
from bento_meta.mdb import MDB

class mdb_update():
    """Mixin for MDB containing database-write methods."""

    # NODES #
    
    @staticmethod
    def _do_update_node(tx, neo4jquery, nid, nhandle):
        result = []

        answers = tx.run(neo4jquery, nid=nid, nhandle=nhandle)

        for record in answers:
            result.append(record)
        return result

    def get_query_to_update_node(self):
        return '''
                MATCH (n1:node)
                WHERE n1.nanoid = $nid and n1._to IS NULL
                CALL apoc.refactor.cloneNodesWithRelationships([n1])
                YIELD input, output
                MATCH (n2:node)
                where n2.nanoid = $nid and n2._to IS NULL and id(n1) <> id(n2)
                SET n1._to = ( n1._from + 1 ), n2._from = (n1._from + 1), n2.handle = $nhandle
                RETURN id(n2)
                '''

    def update_node_by_id(self, nid, nhandle):
        with self.driver.session() as session:
            pprint.pprint('trying to run update node on {} and {}'.format(nid, nhandle))
            query = self.get_query_to_update_node()
            node_ = session.write_transaction(self._do_update_node, query, nid, nhandle)
            # TODO update elasticsearch, remove old handle and add new
        return node_

    
    # ------------------------------------------------------------------------- #

    @staticmethod
    def _do_deprecate_node(tx, neo4jquery, nid):
        print("deprecating node {}".format(nid))
        result = []

        answers = tx.run(neo4jquery, nid=nid)

        for record in answers:
            result.append(record)
        return result

    def get_query_to_deprecate_node(self):
        return '''
                MATCH (n:node)
                WHERE n.nanoid = $nid and n._to IS NULL
                SET n._to = ( n._from + 1 )
                RETURN id(n);
                '''

    def deprecate_node(self, nid):
        with self.driver.session() as session:
            query = self.get_query_to_deprecate_node()
            node_ = session.write_transaction(self._do_deprecate_node, query, nid)
            # TODO update elasticsearch, remove old node
        return node_

    # TERMS #
    
    @staticmethod
    def _do_update_term(tx, neo4jquery, tid, tvalue):
        print("working with tid {} for {}".format(tid, tvalue))
        result = []

        answers = tx.run(neo4jquery, tid=tid, tvalue=tvalue)

        for record in answers:
            # row = {record["id(t2)"]: record["handle"]}
            result.append(record)
        return result

    def get_query_to_update_term(self):
        return '''
                MATCH (t1:term)
                WHERE t1.nanoid = $tid and t1._to IS NULL
                CALL apoc.refactor.cloneNodesWithRelationships([t1])
                YIELD input, output
                MATCH (t2:term)
                where t2.nanoid = $tid and t2._to IS NULL and id(t1) <> id (t2)
                SET t1._to = ( t1._from + 1 ), t2._from = (t1._from + 1), t2.value = $tvalue
                RETURN id(t2)
                '''

    def update_term_by_id(self, tid, tvalue):
        with self.driver.session() as session:
            query = self.get_query_to_update_term()
            term_ = session.write_transaction(self._do_update_term, query, tid, tvalue)
            # TODO update elasticsearch, remove old term and add new term
        return term_

    # ------------------------------------------------------------------------- #

    @staticmethod
    def _do_create_term(tx, neo4jquery, tvalue):
        print("working with for {}".format(tvalue))
        result = []

        answers = tx.run(neo4jquery, tvalue=tvalue)

        for record in answers:
            result.append(record)
        return result

    def get_query_to_create_term(self):
        return '''
                MATCH (t1:term)
                WHERE t1.value = $tvalue and t1._to IS NULL
                SET t1._to = ( t1._from + 1 ), t2._from = (t1._from + 1), t2.value = $tvalue
                RETURN id(t2);
                '''

    def create_term(self, tvalue):
        with self.driver.session() as session:
            query = self.get_query_to_create_term()
            term_ = session.write_transaction(self._do_create_term, query, tvalue)
            # TODO update elasticsearch add new term
        return term_

    # ------------------------------------------------------------------------- #

    @staticmethod
    def _do_deprecate_term(tx, neo4jquery, tid):
        print("deprecating term {}".format(tid))
        result = []

        answers = tx.run(neo4jquery, tid=tid)

        for record in answers:
            result.append(record)
        return result

    def get_query_to_deprecate_term(self):
        return '''
                MATCH (t1:term)
                WHERE t1.nanoid = $tid and t1._to IS NULL
                SET t1._to = ( t1._from + 1 )
                RETURN id(t1);
                '''

    def deprecate_term(self, tid):
        with self.driver.session() as session:
            query = self.get_query_to_deprecate_term()
            term_ = session.write_transaction(self._do_deprecate_term, query, tid)
            # TODO update elasticsearch, remove old term
        return term_

    # PROPERTIES #

    @staticmethod
    def _do_update_property(tx, neo4jquery, pid, phandle):
        print("working with pid {} for {}".format(pid, phandle))
        result = []

        answers = tx.run(neo4jquery, pid=pid, phandle=phandle)

        for record in answers:
            # row = {record["id(t2)"]: record["handle"]}
            result.append(record)
        return result

    def get_query_to_update_property(self):
        return '''
                MATCH (p1:property)
                WHERE p1.nanoid = $pid AND p1._to IS NULL
                CALL apoc.refactor.cloneNodesWithRelationships([p1])
                YIELD input, output
                MATCH (p2:property)
                where p2.nanoid = $pid and p2._to IS NULL and id(p1) <> id (p2)
                SET p1._to = ( p1._from + 1 ), p2._from = (p1._from + 1), p2.handle = $phandle
                RETURN id(p2)
                '''

    def update_property_by_id(self, pid, phandle):
        with self.driver.session() as session:
            query = self.get_query_to_update_property()
            property_ = session.write_transaction(self._do_update_property, query, pid, phandle)
            # TODO update elasticsearch, remove old term and add new term
        return property_

    # ------------------------------------------------------------------------- #

    @staticmethod
    def _do_create_property(tx, neo4jquery, phandle):
        print("working with for {}".format(phandle))
        result = []

        answers = tx.run(neo4jquery, phandle=phandle)

        for record in answers:
            result.append(record)
        return result

    def get_query_to_create_property(self):
        return '''
                MATCH (p1:property)
                WHERE p1.handle = $phandle and p1._to IS NULL
                SET p1._to = ( p1._from + 1 ), p2._from = (p1._from + 1), p2.handle = $phandle
                RETURN id(p2);
                '''

    def create_property(self, phandle):
        with self.driver.session() as session:
            query = self.get_query_to_create_property()
            property_ = session.write_transaction(self._do_create_property, query, phandle)
            # TODO update elasticsearch add new term
        return property_

    # ------------------------------------------------------------------------- #

    @staticmethod
    def _do_deprecate_property(tx, neo4jquery, pid):
        print("deprecating property {}".format(pid))
        result = []

        answers = tx.run(neo4jquery, pid=pid)

        for record in answers:
            result.append(record)
        return result

    def get_query_to_deprecate_property(self):
        return '''
                MATCH (p:property)
                WHERE p.nanoid = $pid and p._to IS NULL
                SET p._to = ( p._from + 1 )
                RETURN id(p);
                '''

    def deprecate_property(self, pid):
        with self.driver.session() as session:
            query = self.get_query_to_deprecate_property()
            property_ = session.write_transaction(self._do_deprecate_property, query, pid)
            # TODO update elasticsearch, remove old term
        return property_

    # TAGS #

    def create_submitter_tag_for_model(self, model=None, tag=None):
        with self.driver.session() as session:
            if (model is not None) and (tag is not None):
                tag_records = session.read_transaction(self._create_submitter_tag_for_model, model, tag)
                # formatted_tags = self.format_tags_records(tag_records)
        return tag_records


    @staticmethod
    def _create_submitter_tag_for_model(tx, model=None, tag=None):
        result = None

        current_app.logger.warn('getting for tag {} in model {}'.format(model, tag))

        create_tag_query_ = """
                MERGE (t:tag {model:$model, key:"submitter", value:$tag})
                RETURN id(t);
                """

        db_records = tx.run(create_tag_query_, model=model, tag=tag)
        current_app.logger.warn('..point BB ... using model {}'.format(model))
        result = db_records.single()[0]

        return result



    def add_submitter_tag_for_model_prop(self, model=None, nodenanoid=None, propnanoid=None, tag=None):
        with self.driver.session() as session:
            if (model is not None) and (tag is not None) and (nodenanoid is not None) and (propnanoid is not None):
                tag_records = session.read_transaction(self._add_submitter_tag_for_model, model, nodenanoid, propnanoid, tag)
                tag_records = session.read_transaction(self._add_submitter_tag_for_prop, model, nodenanoid, propnanoid, tag)
                # formatted_tags = self.format_tags_records(tag_records)
        return tag_records


    @staticmethod
    def _add_submitter_tag_for_model(tx, model=None, nodenanoid=None, propnanoid=None, tag=None):
        result = None

        current_app.logger.warn('ADD tag {} for node {} prop {} in model {}'.format(model, nodenanoid, propnanoid, tag))

        add_tag_query_ = """
                MATCH (n:node { model:$model, nanoid:$nodenanoid })-[np:has_property]->(p:property { nanoid:$propnanoid, model:$model })
                WHERE n._to IS NULL AND p._to IS NULL
                MATCH (t:tag {key:'submitter', model:$model, value:$tag})
                MERGE (p)-[r:has_tag]->(t);
                """
                #MERGE (p)-[r:has_tag]->(t);

        db_records = tx.run(add_tag_query_, model=model, nodenanoid=nodenanoid, propnanoid=propnanoid, tag=tag)
        current_app.logger.warn('..adding tag ... to model {}'.format(model))
        #result = db_records.single()[0]

        return        

    @staticmethod
    def _add_submitter_tag_for_prop(tx, model=None, nodenanoid=None, propnanoid=None, tag=None):
        result = None

        current_app.logger.warn('ADD tag {} for node {} prop {} in model {}'.format(model, nodenanoid, propnanoid, tag))

        add_tag_query_ = """
                MATCH (n:node { model:$model, nanoid:$nodenanoid })-[np:has_property]->(p:property { nanoid:$propnanoid, model:$model })
                -[r:has_tag]->(t:tag {key:'submitter', model:$model, value:$tag})
                WHERE n._to IS NULL AND p._to IS NULL
                MERGE (n)-[z:in_dataset { dataset: $tag } ]->(p);
                """
                #MERGE (p)-[r:has_tag]->(t);

        db_records = tx.run(add_tag_query_, model=model, nodenanoid=nodenanoid, propnanoid=propnanoid, tag=tag)
        current_app.logger.warn('..adding tag ... to prop {}'.format(model))
        #result = db_records.single()[0]

        return   





    def remove_submitter_tag_for_model_prop(self, model=None, nodenanoid=None, propnanoid=None, tag=None):
        with self.driver.session() as session:
            if (model is not None) and (tag is not None) and (nodenanoid is not None) and (propnanoid is not None):
                tag_records = session.read_transaction(self._remove_submitter_tag_for_model_prop, model, nodenanoid, propnanoid, tag)
                # formatted_tags = self.format_tags_records(tag_records)
        return tag_records

    @staticmethod
    def _remove_submitter_tag_for_model_prop(tx, model=None, nodenanoid=None, propnanoid=None, tag=None):
        result = None

        current_app.logger.warn('REMOVING for tag {} for node {} prop {} in model {}'.format(model, nodenanoid, propnanoid, tag))

        count_nodetag_query_ = """
                MATCH (t:tag { value: $tag })<-[r:has_tag]-(p:property {nanoid: $propnanoid })<-[np:has_property]-(n:node)
                WHERE n._to IS NULL AND p._to IS NULL
                with p,count(n) as rels
                return rels;
                """

        db_records = tx.run(count_nodetag_query_, model=model, nodenanoid=nodenanoid, propnanoid=propnanoid, tag=tag)
        result = db_records.single()[0]
        current_app.logger.warn('..found {} counts - for removing tag from model'.format(result))

        ''' only remove p-n tag '''
        if result == 1:
            remove_both_query_ = """
                    MATCH (n:node { model:$model, nanoid:$nodenanoid })-[z:in_dataset { dataset: $tag } ]->(p:property { nanoid:$propnanoid, model:$model })-[r:has_tag]->(t:tag {key:'submitter', model:$model, value:$tag}) 
                    WHERE n._to IS NULL AND p._to IS NULL
                    DELETE r, z
                    """

            db_records = tx.run(remove_both_query_, model=model, nodenanoid=nodenanoid, propnanoid=propnanoid, tag=tag)
            current_app.logger.warn('..removing tag from model {}'.format(model))

        ''' only remove p-n tag '''
        if result > 1:
            remove_node_query_ = """
                    MATCH (n:node { model:$model, nanoid:$nodenanoid })-[z:in_dataset { dataset: $tag } ]->(p:property { nanoid:$propnanoid, model:$model })-[r:has_tag]->(t:tag {key:'submitter', model:$model, value:$tag}) 
                    WHERE n._to IS NULL AND p._to IS NULL
                    DELETE z
                    """

            db_records = tx.run(remove_node_query_, model=model, nodenanoid=nodenanoid, propnanoid=propnanoid, tag=tag)
            current_app.logger.warn('..removing tag from model {}'.format(model))

        return



import os
from bento_sts import config_sts
import yaml
from importlib_resources import files

src = files(config_sts).joinpath("query_paths.yml")
with src.open('r') as fh:
    qp = yaml.load(fh, Loader=yaml.CLoader)

class Config(object):
    SECRET_KEY = os.environ.get("SECRET_KEY") or "supposedly-random-passphrase"
    NEO4J_MDB_URI = os.environ.get("NEO4J_MDB_URI")
    NEO4J_MDB_USER = os.environ.get("NEO4J_MDB_USER")
    NEO4J_MDB_PASS = os.environ.get("NEO4J_MDB_PASS")
    FLASK_LOGFILE = os.environ.get("STS_LOGFILE") or "sts.log"
    LANGUAGES = ["en"]
    MODEL_LIST = [] # set by MDB query in sts.py
    HITS_PER_PAGE = 25
    MAX_ENTS_PER_REQ = 500
    MAX_CONTENT_LENGTH = 2 * 1024 * 1024
    JSONIFY_PRETTYPRINT_REGULAR = True
    SHOW_SINGLE_PAGE = True
    EDITING_FORMS = False
    QUERY_PATHS = qp
    WTF_CSRF_CHECK_DEFAULT = False




# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['bento_sts',
 'bento_sts.api',
 'bento_sts.config_sts',
 'bento_sts.errors',
 'bento_sts.main']

package_data = \
{'': ['*'], 'bento_sts': ['static/*', 'templates/*', 'templates/errors/*']}

install_requires = \
['Bootstrap-Flask>=1.5.2',
 'Flask-Moment>=0.5.2',
 'Flask-Paginate>=2021.10.29',
 'Flask-WTF>=0.14.3',
 'Jinja2>=2.11.3',
 'MarkupSafe>=1.1.1',
 'PyYAML>=5.4.1',
 'WTForms-Components>=0.10.5',
 'WTForms>=2.1',
 'Werkzeug>=1.0.1',
 'bento-meta>=0.0.32',
 'guess-language-spirit>=0.5.3',
 'gunicorn>=20.1.0',
 'idna>=2.6',
 'importlib_resources>=5.4.0',
 'itsdangerous>=0.24',
 'neo4j>=4.1',
 'python-dateutil>=2.6.1',
 'python-dotenv>=0.15.0',
 'python-editor>=1.0.3',
 'pytz>=2017.2',
 'requests>=2.20.0',
 'six>=1.15.0',
 'urllib3>=1.26.5',
 'visitor>=0.1.3']

setup_kwargs = {
    'name': 'bento-sts',
    'version': '0.1.2',
    'description': 'Bento Simple Terminology Server',
    'long_description': '# Welcome to bento-sts\n\nThis is a flask-based implementation of the Simple Terminology Service (STS) for the Bento Metadatabase (MDB).\n\n## Install\n\n\t$ git clone https://github.com/CBIIT/bento-sts.git\n\t$ cd pysts\n\t$ virtualenv sts-venv\n\t$ source sts-venv/bin/activate\n\t$ pip install -r requirements.txt\n\t$ flask run\n\n## Dependencies\n\n`bento-sts` requires a Neo4j-based [MDB](https://github.com/CBIIT/bento-meta).\n\n\n\n',
    'author': 'Mark Benson',
    'author_email': 'mark.benson@nih.gov',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



import os
from flask import url_for, current_app
from bento_meta.mdb import SearchableMDB
from bento_meta.model import Model
from bento_meta.object_map import ObjectMap


plural = {
    "node": "nodes",
    "edge": "edges",
    "relationship": "relationships",
    "property": "properties",
    "tag": "tags",
    "origin": "origins",
    "predicate": "predicates",
    "value_set": "value_sets",
    "term": "terms",
    }

class mdb():
    """Read functionality for driving STS UI. Mixins mdb_update and mdb_tags
could be used here for write and tag functionality."""
    def __init__(self):
        self.mdb = SearchableMDB(current_app.config["NEO4J_MDB_URI"],
                       user=current_app.config["NEO4J_MDB_USER"],
                       password=current_app.config["NEO4J_MDB_PASS"])

    def close(self):
        self.mdb.close()

    def get_list_of_models(self):
        models = self.mdb.get_model_nodes()
        if models:
            return [x["m"] for x in models]
        else:
            return []

    """
    In [4]: m.get_model_by_name('ICDC')
    Out[4]: <bento_meta.model.Model at 0x110378450>
    """
    def get_model_by_name(self, name):
        ObjectMap.clear_cache()
        model = Model(name, self.mdb)
        model_node = self.mdb.get_model_nodes(model=name)
        # if you dont call dget, it wont be populated...
        model.dget()
        model.repository = model_node[0]["m"]["repository"]
        return model

    # ####################################################################### #
    # NODES
    # ####################################################################### #
    def get_node_by_id(self, nid, model=None):
        props_result = self.mdb.get_node_and_props_by_node_id(nid)
        edges_result = self.mdb.get_node_edges_by_node_id(nid)
        to_nodes = {}
        from_nodes = {}
        if not props_result and not edges_result:
            return {}
        result = {"id": nid,
                  "handle": props_result[0]["handle"],
                  "model": props_result[0]["model"],
                  "has_properties": [],
                  "has_relationship_to_nodes": [],
                  "has_relationship_from_nodes": []}
        if (props_result):
            for p in props_result[0]["props"]:
                result["has_properties"].append({
                    "id": p["nanoid"],
                    "handle": p["handle"],
                    "type": "property",
                    "link": url_for("main.entities", entities='properties',
                                    id=p["nanoid"]),
                    "value_domain": p["value_domain"]
                    })
        if (edges_result):
            for e in edges_result:
                node = e["far_node"]
                if (node):
                    rln = e["rln"]
                    add_to = None
                    if e["far_type"] == "has_src":
                        add_to = from_nodes
                    elif e["far_type"] == "has_dst":
                        add_to = to_nodes
                    else:
                        RuntimeError("What?")
                    add_to[node["nanoid"]] = {
                        "id": node["nanoid"],
                        "handle": node["handle"],
                        "link": url_for("main.entities", entities='nodes',
                                        id=node["nanoid"]),
                        "type": "node",
                        "relationship": {
                            "id": rln["nanoid"],
                            "handle": rln["handle"],
                            "link": "/relationships/{}".format(rln["nanoid"]),
                            "type": "relationship"
                        }
                    }
            result["has_relationship_to_nodes"].extend(to_nodes.values())
            result["has_relationship_from_nodes"].extend(from_nodes.values())
        return result

    # ----------------------------------------------------------------------- #

    def get_list_of_nodes(self, model=None):
        """
        In [3]: m.get_list_of_nodes()
        Out[3]:
        [{'yXWr0Y': 'study_site'},
         {'N0Qx7Z': 'off_study'},
         {'nUoHJH': 'diagnosis
        """
        result = self.mdb.get_nodes_by_model(model)
        if result:
            return [(x["nanoid"], x["handle"], x["model"]) for x in result]
        else:
            return []

    # ####################################################################### #
    # VALUESETS
    # ####################################################################### #

    def get_valueset_by_id(self, vsid, model=None):
        vs_result = self.mdb.get_valueset_by_id(vsid)
        if not vs_result:
            return {}
        result = {
            "id": vs_result[0]["nanoid"],
            "handle": vs_result[0]["handle"],
            "model": model,  # kludge
            "url": vs_result[0]["url"],
            "desc": vs_result[0]["desc"],
            "type": "valueset",
            "link": url_for("main.entities", entities='valuesets',
                            id=vs_result[0]["nanoid"]),
            "_for_propertyhandle": "_for_propertyhandle",  # kludge
            "_for_propertyid": "_for_propertyid"  # kludge
            }
        result["has_property"] = [{"id": x["nanoid"], "handle": x["handle"],
                                   "type": "property", "model": x["model"],
                                   "link": "/properties/{}".format(x["nanoid"])}
                                  for x in vs_result[0].props]
        result["has_terms"] = [{"id": x["nanoid"], "value": x["value"],
                                "type": "term",
                                "link":url_for('main.entities', entities='terms',
                                               id=x["nanoid"])}
                               for x in vs_result[0].terms]
        if model:
            result["has_property"] = [x for x in result["has_property"]
                                      if x["model"] == model]
        # kludge - original assumption is that a valueset has only one
        # associated property, but this is not necessarily true.
        if result["has_property"]:
            result["_for_propertyhandle"] = result["has_property"][0]["handle"]
            result["_for_propertyid"] = result["has_property"][0]["nanoid"]
        return result

    # ----------------------------------------------------------------------- #

    def get_list_of_valuesets(self, model=None):
        vs_result = self.mdb.get_valuesets_by_model(model)
        if not vs_result:
            return []
        # kludge assuming that there is only one property assoc with each
        # valueset:
        return [{"id": x["value_set"]["nanoid"],
                 "handle": x["props"][0]["handle"]}
                for x in vs_result]

    # ####################################################################### #
    # TERMS
    # ####################################################################### #
    def get_term_by_id(self, tid):
        t_result = self.mdb.get_term_by_id(tid)
        if not t_result:
            return {}
        result = t_result[0]["term"]
        result["id"] = result["nanoid"]
        del result["nanoid"]
        result["type"] = "term"
        result["link"] = url_for("main.entities", entities='terms',
                                 id=result["id"],
                                 _external=False)
        result["has_origin"] = {}
        origin = t_result[0]["origin"]
        if origin:
            origin["id"] = origin["nanoid"]
            origin["type"] = "origin"
            origin["link"] = "/origins/{}".format(origin["nanoid"])
            result["has_origin"] = origin
        return result

    # ======================================================================= #

    def get_list_of_terms(self, model=None):
        t_result = self.mdb.get_props_and_terms_by_model(model)
        result = []
        for p in t_result:
            result.extend([{"id": x["nanoid"], "value": x["value"] if "value" in x else None,
                            "property": p["prop"]["handle"],
                            "model": p["prop"]["model"],
                            "propid": p["prop"]["nanoid"]}
                           for x in p["terms"]])
        return result

    # ####################################################################### #
    # ORIGINS
    # ####################################################################### #

    def get_list_of_origins(self, dummy=None):
        origins = self.mdb.get_origins()
        result = [{x["o"]["nanoid"]:x["o"]["name"]}
                  for x in origins]
        if result:
            return result
        else:
            return []

    def get_origin_by_id(self, oid):
        result = self.mdb.get_origin_by_id(oid)
        return result

    # ####################################################################### #
    # PROPERTIES
    # ####################################################################### #

    def get_list_of_properties(self, model=None):
        np_result = self.mdb.get_nodes_and_props_by_model(model)
        if not np_result:
            return []
        result = []
        for np in np_result:
            result.extend([(p["nanoid"], p["handle"],
                            np["model"], np["handle"], np["id"])
                           for p in np["props"]])
        return result

    def get_property_by_id(self, pid, model=None):
        p_result = self.mdb.get_prop_node_and_domain_by_prop_id(pid)
        if not p_result:
            return {}
        pr = p_result[0]
        result = {
            "model": pr["model"],
            "prop": pr["prop"],
            "type": "property",
            "link": "/properties/{}".format(pr["id"]),
            "_for_nodehandle": pr["node"]["handle"],
            "_for_nodeid": pr["node"]["nanoid"]
            }
        if pr["value_set"]:
            result["has_valueset"] = {
                "id": pr["value_set"]["nanoid"],
                "type": "valueset",
                "link": url_for("main.entities", entities='valuesets',
                                id=pr["value_set"]["nanoid"])
                }
        if pr["terms"]:
            result["has_terms"] = [{"id": t["nanoid"], "value": t["value"],
                                    "type": "term",
                                    "link": url_for("main.entities",
                                                    entities='terms',
                                                    id=t["nanoid"])}
                                   for t in pr["terms"]]
        return result

    # TAGS
    
    def get_tagged_entities(self, tag_key, tag_value=None, model=None):
        ents_by_tag = self.mdb.get_entities_by_tag(tag_key, tag_value, model)
        for i in range(0, len(ents_by_tag)):
            ents_by_tag[i]['plural'] = plural[ents_by_tag[i]['entity']]
        return ents_by_tag

    def get_tags_and_values(self):
        tags = self.mdb.get_tags_and_values()
        return tags

    # SEARCH

    def search_entity_handles(self, qstring):
        return self.mdb.search_entity_handles(qstring)

    def search_terms(self, qstring, search_values=True,
                     search_definitions=True):
        return self.mdb.search_terms(qstring, search_values=search_values,
                                     search_definitions=search_definitions)



from flask import request
from flask_wtf import FlaskForm
from wtforms import StringField, SubmitField, TextAreaField, SelectField
from wtforms.validators import ValidationError, DataRequired, Length, InputRequired

class SearchForm(FlaskForm):
    qstring = StringField("Query", validators=[DataRequired()])
    terms = SubmitField('Search Terms')
    models = SubmitField('Search Models')
    def __init__(self, *args, **kwargs):
        if "formdata" not in kwargs:
            kwargs["formdata"] = request.args
        if "csrf_enabled" not in kwargs:
            kwargs["csrf_enabled"] = False
        super(SearchForm, self).__init__(*args, **kwargs)

class SelectModelForm(FlaskForm):
    model = SelectField('Model')
    filter = SubmitField('Filter')
    export = SubmitField('Export JSON')




        



# coding: utf-8

from __future__ import absolute_import
from datetime import date, datetime  # noqa: F401

from typing import List, Dict  # noqa: F401

from swagger_server.models.base_model_ import Model
from swagger_server import util


class ModelProperty(Model):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    def __init__(self, handle: str=None, model: str=None, nanoid: str=None, is_required: str=None, value_domain: str=None, units: str=None, pattern: str=None):  # noqa: E501
        """ModelProperty - a model defined in Swagger

        :param handle: The handle of this ModelProperty.  # noqa: E501
        :type handle: str
        :param model: The model of this ModelProperty.  # noqa: E501
        :type model: str
        :param nanoid: The nanoid of this ModelProperty.  # noqa: E501
        :type nanoid: str
        :param is_required: The is_required of this ModelProperty.  # noqa: E501
        :type is_required: str
        :param value_domain: The value_domain of this ModelProperty.  # noqa: E501
        :type value_domain: str
        :param units: The units of this ModelProperty.  # noqa: E501
        :type units: str
        :param pattern: The pattern of this ModelProperty.  # noqa: E501
        :type pattern: str
        """
        self.swagger_types = {
            'handle': str,
            'model': str,
            'nanoid': str,
            'is_required': str,
            'value_domain': str,
            'units': str,
            'pattern': str
        }

        self.attribute_map = {
            'handle': 'handle',
            'model': 'model',
            'nanoid': 'nanoid',
            'is_required': 'is_required',
            'value_domain': 'value_domain',
            'units': 'units',
            'pattern': 'pattern'
        }

        self._handle = handle
        self._model = model
        self._nanoid = nanoid
        self._is_required = is_required
        self._value_domain = value_domain
        self._units = units
        self._pattern = pattern

    @classmethod
    def from_dict(cls, dikt) -> 'ModelProperty':
        """Returns the dict as a model

        :param dikt: A dict.
        :type: dict
        :return: The Property of this ModelProperty.  # noqa: E501
        :rtype: ModelProperty
        """
        return util.deserialize_model(dikt, cls)

    @property
    def handle(self) -> str:
        """Gets the handle of this ModelProperty.


        :return: The handle of this ModelProperty.
        :rtype: str
        """
        return self._handle

    @handle.setter
    def handle(self, handle: str):
        """Sets the handle of this ModelProperty.


        :param handle: The handle of this ModelProperty.
        :type handle: str
        """
        if handle is None:
            raise ValueError("Invalid value for `handle`, must not be `None`")  # noqa: E501

        self._handle = handle

    @property
    def model(self) -> str:
        """Gets the model of this ModelProperty.


        :return: The model of this ModelProperty.
        :rtype: str
        """
        return self._model

    @model.setter
    def model(self, model: str):
        """Sets the model of this ModelProperty.


        :param model: The model of this ModelProperty.
        :type model: str
        """
        if model is None:
            raise ValueError("Invalid value for `model`, must not be `None`")  # noqa: E501

        self._model = model

    @property
    def nanoid(self) -> str:
        """Gets the nanoid of this ModelProperty.


        :return: The nanoid of this ModelProperty.
        :rtype: str
        """
        return self._nanoid

    @nanoid.setter
    def nanoid(self, nanoid: str):
        """Sets the nanoid of this ModelProperty.


        :param nanoid: The nanoid of this ModelProperty.
        :type nanoid: str
        """
        if nanoid is None:
            raise ValueError("Invalid value for `nanoid`, must not be `None`")  # noqa: E501

        self._nanoid = nanoid

    @property
    def is_required(self) -> str:
        """Gets the is_required of this ModelProperty.


        :return: The is_required of this ModelProperty.
        :rtype: str
        """
        return self._is_required

    @is_required.setter
    def is_required(self, is_required: str):
        """Sets the is_required of this ModelProperty.


        :param is_required: The is_required of this ModelProperty.
        :type is_required: str
        """

        self._is_required = is_required

    @property
    def value_domain(self) -> str:
        """Gets the value_domain of this ModelProperty.


        :return: The value_domain of this ModelProperty.
        :rtype: str
        """
        return self._value_domain

    @value_domain.setter
    def value_domain(self, value_domain: str):
        """Sets the value_domain of this ModelProperty.


        :param value_domain: The value_domain of this ModelProperty.
        :type value_domain: str
        """
        if value_domain is None:
            raise ValueError("Invalid value for `value_domain`, must not be `None`")  # noqa: E501

        self._value_domain = value_domain

    @property
    def units(self) -> str:
        """Gets the units of this ModelProperty.


        :return: The units of this ModelProperty.
        :rtype: str
        """
        return self._units

    @units.setter
    def units(self, units: str):
        """Sets the units of this ModelProperty.


        :param units: The units of this ModelProperty.
        :type units: str
        """

        self._units = units

    @property
    def pattern(self) -> str:
        """Gets the pattern of this ModelProperty.


        :return: The pattern of this ModelProperty.
        :rtype: str
        """
        return self._pattern

    @pattern.setter
    def pattern(self, pattern: str):
        """Sets the pattern of this ModelProperty.


        :param pattern: The pattern of this ModelProperty.
        :type pattern: str
        """

        self._pattern = pattern



import connexion
import six

from swagger_server import util


def tag_key_value_entities_count_get(key, value, skip=None, limit=None):  # noqa: E501
    """Get number of entities tagged by key:value

     # noqa: E501

    :param key: Tag node key (string)
    :type key: str
    :param value: Tag node value (string)
    :type value: str
    :param skip: Pagination - number of items to skip 
    :type skip: int
    :param limit: Pagination - number of items to return 
    :type limit: int

    :rtype: object
    """
    return 'do some magic!'


def tag_key_value_entities_get(key, value):  # noqa: E501
    """Get list of entities tagged by key:value

     # noqa: E501

    :param key: Tag node key (string)
    :type key: str
    :param value: Tag node value (string)
    :type value: str

    :rtype: object
    """
    return 'do some magic!'


def tag_key_values_get(key):  # noqa: E501
    """Get list of tags having specified tag key

     # noqa: E501

    :param key: Tag node key (string)
    :type key: str

    :rtype: object
    """
    return 'do some magic!'



import sys
print("main routes", file=sys.stderr)
import re
from flask import (
    render_template,
    flash,
    redirect,
    url_for,
    request,
    g,
    jsonify,
    current_app,
    Response,
    abort,
)
from flask_paginate import Pagination, get_page_parameter
from .forms import SearchForm, SelectModelForm
# import app.search
from . import bp
from ..util import get_yaml_for
from ..mdb import mdb
# from app.arc import diff_mdf



@bp.before_app_request
def before_request():
    g.search_form = SearchForm()
    g.locale = 'EN_US'


@bp.route('/', methods=['GET', 'POST'])
@bp.route("/index", methods=["GET", "POST"])
def index():
    return render_template(
        "index.html",
        title="Home",
    )


# - - - - - - - - - - - - - - - - - - - - - - - - - - - - -


@bp.route("/models/<name>")
@bp.route("/models")
def models(name=None):
    format = request.args.get("format")
    m = mdb()

    if name is not None:
        model_ = m.get_model_by_name(name)

        if format == 'yaml':
            yaml = get_yaml_for(model_.handle)
            return Response(yaml, mimetype='text/plain')

        else:
            return render_template(
                "mdb-model.html",
                title="Model: {}".format(model_.handle),
                mdb=model_,
                subtype="main.models",
                display="detail",
            )

    else:
        models_ = m.get_list_of_models()
        return render_template(
            "mdb-model.html",
            title="Models",
            mdb=sorted(models_, key=lambda x:x["name"]),
            subtype="main.models",
            display="list",
        )

@bp.route("/<entities>", defaults={'id': None}, methods=['GET','POST'])
@bp.route('/<entities>/<id>', methods=['GET','POST'])

def entities(entities, id):

    if request.form.get("format"):
        format = request.args.get("format")
    elif request.form.get("export"):
        format = "json"
    else:
        format = ""

    model = request.args.get("model") or request.form.get("model") or "All"
    id_ = request.args.get("id")
    page = request.args.get(get_page_parameter(), type=int, default=1)
    select_form = SelectModelForm()
    select_form.model.choices = [(x['handle'],x['handle']) for x
                                     in current_app.config["MODEL_LIST"]]
    current_app.logger.info("model: {}, page: {}, format: {}".format(model, page, format))
    id = id or id_
    m = mdb()
    dispatch = {
        "nodes": {
            "title": "Node",
            "list_title": "Nodes",
            "template": "mdb-node.html",
            "subtype": "nodes",
            "sort_key": lambda x: x[1],
            "get_by_id": m.get_node_by_id,
            "get_list":m.get_list_of_nodes,
        },
        "properties": {
            "title": "Property",
            "list_title": "Properties",
            "template": "mdb-property.html",
            "subtype": "properties",
            "sort_key": lambda x: (x[1], x[2], x[3]),
            "display": "prop-tuple",
            "get_by_id": m.get_property_by_id,
            "get_list": m.get_list_of_properties,
        },
        "valuesets": {
            "title": "Value Set",
            "list_title": "Value Sets",
            "template": "mdb-valueset.html",
            "subtype": "valuesets",
            "sort_key": lambda x: x["handle"],
            "display": "list",
            "get_by_id": m.get_valueset_by_id,
            "get_list": m.get_list_of_valuesets,
        },
        "terms": {
            "title": "Term",
            "list_title": "Terms",
            "template": "mdb-term.html",
            "subtype": "terms",
            "sort_key": lambda x: (x["model"], x["property"], x["value"] if type(x["value"]) == str else ""),
            "display": "term-tuple",
            "get_by_id": m.get_term_by_id,
            "get_list": m.get_list_of_terms,
        },
        "origins": {
            "title": "Origin",
            "list_title": "Origins",
            "template": "mdb-origin.html",
            "subtype": "origins",
            "sort_key": lambda x:list(x.values())[0],
            "display": "list",
            "get_list": m.get_list_of_origins,
            "get_by_id": m.get_origin_by_id
        },
    }

    # A: single entity
    if id is not None:
        ent_ = dispatch[entities]["get_by_id"](id)
        if ent_ is None or not bool(ent_):
            return render_template('/errors/400.html'), 400
        if type(ent_) == dict and ent_.get("has_properties"): # nodes only kludge
            ent_['has_properties'].sort(key=lambda x:x['handle'])
        if format == "json":
            return jsonify(ent_)
        else:
            return render_template(
                dispatch[entities]["template"],
                title=dispatch[entities]["title"],
                mdb=ent_,
                subtype=dispatch[entities]["subtype"],
                display="detail",
            )

    # B: filter by model
    ents_ = dispatch[entities]["get_list"]( None if model == 'All' else model)

    if format == "json":
        return jsonify(ents_)
    else:
        pgurl = url_for('main.entities',entities=entities)
        pgurl += "?page={0}"
        if model:
            pgurl += "&model={}".format(model)
        pagination = Pagination(
            page=page,
            total=len(ents_),
            record_name=entities,
            href=pgurl
            )
        rendered = render_template(
            "mdb.html",
            title=dispatch[entities]["list_title"],
            mdb=sorted(ents_,
                       key=dispatch[entities]["sort_key"]),
            subtype=dispatch[entities]["subtype"],
            display=dispatch[entities].get("display") or "tuple",
            first=(pagination.page-1)*pagination.per_page,
            last=min((pagination.page)*pagination.per_page,len(ents_)),
            pagination=pagination,
            form=select_form,
            model=model
        )
        # get a load of THIS kludge, dude.
        if model:
            rendered = re.sub('option value="{}"'.format(model),
                            'option selected="true" value={}'.format(model),
                            rendered)

        return rendered
# ---------------------------------------------------------------------------

@bp.route("/tags", defaults={'key':None,'value':None},methods=['GET','POST'])
@bp.route("/tags/<key>", methods=['GET','POST'], defaults={'value':None})
@bp.route("/tags/<key>/<value>", methods=['GET','POST'])
def tags(key=None,value=None,model=None):
    key = key or request.args.get("key")
    val = value or request.args.get("value")
    model = model or request.args.get("model")
    ents = []
    m = mdb()
    format = request.args.get("format")
    if request.form.get("format"):
        format = request.args.get("format")
    elif request.form.get("export"):
        format = "json"
    else:
        pass

    if key:
        ents = m.get_tagged_entities(key, val, model)
    else:
        ents = m.get_tags_and_values()

    if format == "json":
        return jsonify(ents)
    else:
        if key:
            return render_template(
                "mdb-tag.html",
                title="Tagged Entities",
                key=key,
                value=val,
                model=model,
                ents=ents,
                display='entities')
        else:
            return render_template(
                "mdb-tag.html",
                title="Tags",
                ents=ents,
                display='tags')

@bp.route("/search")
def search():
    if not g.search_form.validate():
        return redirect(url_for("main.index"))
    m = mdb()
    format = request.args.get("format")
    if request.form.get("format"):
        format = request.args.get("format")
    elif request.form.get("export"):
        format = "json"
    else:
        pass

    qstring = request.args.get("qstring")
    ents = []
    thing = ""
    if request.args.get("terms"):
        ents = m.search_terms(qstring)
        thing = "terms"
    elif request.args.get("models"):
        ents = m.search_entity_handles(qstring)
        thing = "models"
    else:
        abort(400)
    if format == 'json':
        return jsonify(ents)

    pg_tot = 0
    if thing == "terms":
        pg_tot = len(ents)
    elif thing == "models":
        if ents:
            pg_tot = max(len(ents["nodes"]), len(ents["properties"]),
                         len(ents["relationships"]))
    else:
        pass
    pagination = Pagination(
        page=request.args.get("page", 1, type=int),
        record_name="Hits",
        total=pg_tot,
        per_page=current_app.config["HITS_PER_PAGE"],
        )
    first = {}
    last = {}
    nohits = False
    if thing == "terms":
        if ents:
            first["terms"] = (pagination.page-1)*pagination.per_page
            last["terms"] = min(pagination.page*pagination.per_page, len(ents))
        else:
            thing = "no_hits"
    elif thing == "models":
        if ents:
            pp = round(pagination.per_page/3)
            first["nodes"] = min(len(ents["nodes"])-pp,
                                 pagination.page-1*pp)
            last["nodes"] = min(len(ents["nodes"]),
                                pagination.page*pp)
            first["properties"] = min(len(ents["properties"])-pp,
                                      pagination.page-1*pp)
            last["properties"] = min(len(ents["properties"]),
                                     pagination.page*pp)
            first["relationships"] = min(len(ents["relationships"])-pp,
                                         pagination.page-1*pp)
            last["relationships"] = min(len(ents["relationships"]),
                                        pagination.page*pp)
        else:
            thing = "no_hits"
    else:
        raise RuntimeError("Huh???")
    
    return render_template(
        "search.html",
        title="Search",
        ents=ents,
        thing=thing,
        pagination=pagination,
        first=first,
        last=last,
    )

@bp.route("/about-mdb")

def about_mdb():
    return render_template("about-mdb.html", title="About MDB")


@bp.route("/about-sts")

def about_sts():
    return render_template("about-sts.html", title="About STS")

@bp.errorhandler(413)
def too_large(e):
    return "File is too large", 413

@bp.route('/reports', methods=['GET', 'POST'])
def reports():
    return render_template('reports.html',)

@bp.route('/versionhistory', methods=['GET', 'POST'])
def versionhistory():
    return render_template('version-history.html')



import connexion
import six

from swagger_server.models.model_property import ModelProperty  # noqa: E501
from swagger_server.models.node import Node  # noqa: E501
from swagger_server import util


def model_model_handle_node_node_handle_get(modelHandle, nodeHandle):  # noqa: E501
    """Retrieve a specified node from a model

     # noqa: E501

    :param modelHandle: Handle (\\&#39;name\\&#39;) of model (use /models to find available handles) 
    :type modelHandle: str
    :param nodeHandle: Handle (\\&#39;name\\&#39;) of node (use /model/{modelHandle}/nodes to find available nodes 
    :type nodeHandle: str

    :rtype: Node
    """
    return 'do some magic!'


def model_model_handle_node_node_handle_properties_count_get(modelHandle, nodeHandle):  # noqa: E501
    """Get number of  properties for specified node

     # noqa: E501

    :param modelHandle: Handle (\\&#39;name\\&#39;) of model (use /models to find available handles) 
    :type modelHandle: str
    :param nodeHandle: Handle (\\&#39;name\\&#39;) of node (use /model/{modelHandle}/nodes to find available nodes 
    :type nodeHandle: str

    :rtype: object
    """
    return 'do some magic!'


def model_model_handle_node_node_handle_properties_get(modelHandle, nodeHandle, skip=None, limit=None):  # noqa: E501
    """Get all properties for specified node

     # noqa: E501

    :param modelHandle: Handle (\\&#39;name\\&#39;) of model (use /models to find available handles) 
    :type modelHandle: str
    :param nodeHandle: Handle (\\&#39;name\\&#39;) of node (use /model/{modelHandle}/nodes to find available nodes 
    :type nodeHandle: str
    :param skip: Pagination - number of items to skip 
    :type skip: int
    :param limit: Pagination - number of items to return 
    :type limit: int

    :rtype: object
    """
    return 'do some magic!'


def model_model_handle_node_node_handle_property_prop_handle_get(modelHandle, nodeHandle, propHandle):  # noqa: E501
    """Retrieve a specified property from a model

     # noqa: E501

    :param modelHandle: Handle (\\&#39;name\\&#39;) of model (use /models to find available handles) 
    :type modelHandle: str
    :param nodeHandle: Handle (\\&#39;name\\&#39;) of node (use /model/{modelHandle}/nodes to find available nodes 
    :type nodeHandle: str
    :param propHandle: Handle (\\&#39;name\\&#39;) of property (use /model/{modelHandle}/node/{nodeHandle}/properties to find available properties 
    :type propHandle: str

    :rtype: ModelProperty
    """
    return 'do some magic!'


def model_model_handle_node_node_handle_property_prop_handle_term_term_value_get(modelHandle, nodeHandle, propHandle, termValue):  # noqa: E501
    """Retrieve a specified term from a property\\&#39;s acceptable value set 

     # noqa: E501

    :param modelHandle: Handle (\\&#39;name\\&#39;) of model (use /models to find available handles) 
    :type modelHandle: str
    :param nodeHandle: Handle (\\&#39;name\\&#39;) of node (use /model/{modelHandle}/nodes to find available nodes 
    :type nodeHandle: str
    :param propHandle: Handle (\\&#39;name\\&#39;) of property (use /model/{modelHandle}/node/{nodeHandle}/properties to find available properties 
    :type propHandle: str
    :param termValue: String representation (\\&#39;value\\&#39;) of the term (use /model/{modelHandle}/node/{nodeHandle}/property/{propHandle}/terms to find available terms 
    :type termValue: str

    :rtype: object
    """
    return 'do some magic!'


def model_model_handle_node_node_handle_property_prop_handle_terms_count_get(modelHandle, nodeHandle, propHandle):  # noqa: E501
    """Get number of  properties for specified node

     # noqa: E501

    :param modelHandle: Handle (\\&#39;name\\&#39;) of model (use /models to find available handles) 
    :type modelHandle: str
    :param nodeHandle: Handle (\\&#39;name\\&#39;) of node (use /model/{modelHandle}/nodes to find available nodes 
    :type nodeHandle: str
    :param propHandle: Handle (\\&#39;name\\&#39;) of property (use /model/{modelHandle}/node/{nodeHandle}/properties to find available properties 
    :type propHandle: str

    :rtype: object
    """
    return 'do some magic!'


def model_model_handle_node_node_handle_property_prop_handle_terms_get(modelHandle, nodeHandle, propHandle, skip=None, limit=None):  # noqa: E501
    """Get the terms (acceptable values) for specified property, if applicable to property 

     # noqa: E501

    :param modelHandle: Handle (\\&#39;name\\&#39;) of model (use /models to find available handles) 
    :type modelHandle: str
    :param nodeHandle: Handle (\\&#39;name\\&#39;) of node (use /model/{modelHandle}/nodes to find available nodes 
    :type nodeHandle: str
    :param propHandle: Handle (\\&#39;name\\&#39;) of property (use /model/{modelHandle}/node/{nodeHandle}/properties to find available properties 
    :type propHandle: str
    :param skip: Pagination - number of items to skip 
    :type skip: int
    :param limit: Pagination - number of items to return 
    :type limit: int

    :rtype: object
    """
    return 'do some magic!'


def model_model_handle_nodes_count_get(modelHandle):  # noqa: E501
    """Get number of nodes for specified model

     # noqa: E501

    :param modelHandle: Handle (\\&#39;name\\&#39;) of model (use /models to find available handles) 
    :type modelHandle: str

    :rtype: object
    """
    return 'do some magic!'


def model_model_handle_nodes_get(modelHandle, skip=None, limit=None):  # noqa: E501
    """Get all nodes for specified model

     # noqa: E501

    :param modelHandle: Handle (\\&#39;name\\&#39;) of model (use /models to find available handles) 
    :type modelHandle: str
    :param skip: Pagination - number of items to skip 
    :type skip: int
    :param limit: Pagination - number of items to return 
    :type limit: int

    :rtype: object
    """
    return 'do some magic!'
